/*
 * raw1394.h - Raw IEEE-1394 I/O kernel driver header file
 * Copyright (C)1999 Emanuel Pirker <epirker@edu.uni-klu.ac.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __RAW1394H_
#define __RAW1394H_

typedef quadlet_t speedmap_t[4000];
typedef quadlet_t topologymap_t[255];

#define RAW1394_IOC_MAGIC '1'

/* Note that many of the ioctl()s will disappear soon.
   Most can be done via read/write. The library will
   reflect these changes */
#define RAW1394_GET_PHYID         _IOR(RAW1394_IOC_MAGIC,0x2,nodeid_t)
#define RAW1394_GET_IRMID         _IOR(RAW1394_IOC_MAGIC,0x3,nodeid_t)
#define RAW1394_GET_BMGRID        _IOR(RAW1394_IOC_MAGIC,0x4,nodeid_t)
#define RAW1394_GET_NODECOUNT     _IOR(RAW1394_IOC_MAGIC,0x5,int)
#define RAW1394_RESET_BUS         _IO(RAW1394_IOC_MAGIC,0x6)
#define RAW1394_SET_CHANNEL       _IOW(RAW1394_IOC_MAGIC,0x9,int)
#define RAW1394_GET_CHANNEL       _IOR(RAW1394_IOC_MAGIC,0x10,int)
#define RAW1394_ENA_ISO           _IOW(RAW1394_IOC_MAGIC,0x11,int)
#define RAW1394_SET_CYCLE_COUNTER _IOW(RAW1394_IOC_MAGIC,0x12,int)
#define RAW1394_GET_CYCLE_COUNTER _IOR(RAW1394_IOC_MAGIC,0x13,int)
#define RAW1394_SET_BUS_ID        _IOW(RAW1394_IOC_MAGIC,0x14,int)
#define RAW1394_ACT_CYCLE_MASTER  _IOW(RAW1394_IOC_MAGIC,0x15,int)
#define RAW1394_CANCEL_REQUESTS   _IO(RAW1394_IOC_MAGIC,0x16)
#define RAW1394_GET_EVENT_BUFFER  _IOR(RAW1394_IOC_MAGIC,0x17,struct hpsb_event)


#define MAX_BLOCK_QUADLETS       (2048/sizeof(quadlet_t))

/* Magic codes */
#define RAW1394_QUADREAD     0x11111
#define RAW1394_QUADWRITE    0x11112
#define RAW1394_BLOCKREAD    0x11113
#define RAW1394_BLOCKWRITE   0x11114
#define RAW1394_LOCK         0x11115

typedef quadlet_t block_t[MAX_BLOCK_QUADLETS];

struct RAW1394request_t {
    /* request part */
    int type;
    int generation;
    octlet_t destinationOffset;
    nodeid_t destinationID;
    quadlet_t quadletData;
    quadlet_t length;
    int extcode;

    /* response part */
    quadlet_t requestStatus;
    quadlet_t responseCode;

    /* block */
    block_t block;
};

typedef struct RAW1394request_t RAW1394quadread;
typedef struct RAW1394request_t RAW1394quadwrite;
typedef struct RAW1394request_t RAW1394blockread;
typedef struct RAW1394request_t RAW1394blockwrite;

#define RAW1394_BUFFER_SIZE (sizeof(struct RAW1394request_t))

#define MAX_RAW_PORTS 4

#ifdef __KERNEL__

#define RAW1394_MAJOR 171
#define RAW1394_DRIVER_NAME "raw1394"

typedef enum { lck_USED, lck_UNUSED, lck_UNUSABLE } usage_status_t;

struct raw1394_port {
    struct hpsb_host *host;
    usage_status_t usage_status;

    struct hpsb_event event_buf;
    struct fasync_struct *fasync_queue;

    wait_queue_head_t event_wait;

    struct RAW1394request_t req_buf[0];
    int req_num;

    u32 buffer[RAW1394_BUFFER_SIZE];

    wait_queue_head_t rd_q;
    wait_queue_head_t wr_q;

    int dirtybuf;
};

#endif


#endif
