/* -*- linux-c -*- --------------------------------------------------------- *
 *
 * linux/fs/autofs/symlink.c
 *
 *  Copyright 1997-1998 Transmeta Corporation -- All Rights Reserved
 *
 * This file is part of the Linux kernel and is made available under
 * the terms of the GNU General Public License, version 2, or at your
 * option, any later version, incorporated herein by reference.
 *
 * ------------------------------------------------------------------------- */

#include <linux/string.h>
#include <linux/sched.h>
#include "autofs_i.h"

static int autofs_readlink(struct dentry *dentry, char *buffer, int buflen)
{
	struct autofs_info *ino = autofs_dentry_ino(dentry);
	int len;

	len = ino->size;
	if (len > buflen) 
		len = buflen;
	copy_to_user(buffer, ino->u.symlink, len);
	return len;
}

static struct dentry * autofs_follow_link(struct dentry *dentry,
					  struct dentry *base,
					  unsigned int follow)
{
	struct autofs_info *ino = autofs_dentry_ino(dentry);

	return lookup_dentry(ino->u.symlink, base, follow);
}

struct inode_operations autofs_symlink_inode_operations = {
	readlink:	autofs_readlink,
	follow_link:	autofs_follow_link
};
