#include <linux/config.h>
#include <linux/kernel.h>

#include <linux/malloc.h>
#include <linux/fs.h>
#include <linux/unistd.h>
#include <net/sock.h>
#include <linux/socket.h>
#include <linux/skbuff.h>
#include <linux/mount.h>

#include <linux/sunrpc/types.h>
#include <linux/sunrpc/stats.h>
#include <linux/sunrpc/svc.h>
#include <linux/sunrpc/svcsock.h>
#include <linux/nfsd/nfsd.h>
#include <linux/nfsd/stats.h>
#include <linux/nfsd/cache.h>
#include <linux/nfsd/xdr.h>
#include <linux/lockd/bind.h>

#include <linux/netlink.h>

static struct sock *nfsdnlsk = NULL;

	
struct nfsd_netlink_callback {
	int vers;
	struct sockaddr s;
	struct knfs_fh fh;
	int devlen;
	char path[NFS_MAXPATHLEN+1];
};

void nfsd_netlink_init()
{
	nfsdnlsk = netlink_kernel_create(NETLINK_NFSD, NULL);

	if (nfsdnlsk == NULL)
		printk("nfsd_netlink_init: cannot initialise netlink\n");
}

void nfsd_netlink_shutdown()
{
	if (nfsdnlsk) {
		if (nfsdnlsk->socket)
			sock_release(nfsdnlsk->socket);
		nfsdnlsk = NULL;
	}

}


int nfsd_netlink_notify(struct svc_rqst *rqstp, struct svc_fh *fhp)
{
	struct knfs_fh *fh = &fhp->fh_handle;
	struct dentry *d;
	struct sk_buff *outskb;
	struct nfsd_netlink_callback *cbk;
	struct vfsmount *vfsm;
	char *path;

	if (nfsdnlsk == NULL)
		return nfserr_stale;

	vfsm = lookup_vfsmnt(u32_to_kdev_t(fh->fh_base.fb_dev));

	if (!vfsm)
		return nfserr_stale;

	d = find_fh_dentry(fh);
	
	if (!d)
		return nfserr_stale;
	if (IS_ERR(d))
		return nfserrno(-PTR_ERR(d));

	outskb = alloc_skb(sizeof(*cbk), GFP_KERNEL);

	if (!outskb) {
		if (net_ratelimit())
			printk(KERN_WARNING "nfsd_netlink: packet drop due to"
			       "netlink failure\n");
		return nfserr_stale;
	}

	skb_put(outskb, sizeof(*cbk));

	cbk = (struct nfsd_netlink_callback *)outskb->data;

	cbk->vers = 1;
	memcpy(&cbk->s, &rqstp->rq_addr, sizeof(struct sockaddr));
	cbk->fh= *fh;

	path = d_path(d, cbk->path, NFS_MAXPATHLEN + 1);
	memmove (cbk->path, path,
		 &cbk->path[NFS_MAXPATHLEN] - path + 1);
	cbk->devlen = strlen(cbk->path);

	if (NFS_MAXPATHLEN - cbk->devlen > 1)
		strncpy(&cbk->path[cbk->devlen+1], vfsm->mnt_dirname,
			NFS_MAXPATHLEN - cbk->devlen);

	netlink_broadcast(nfsdnlsk, outskb, 0, ~0, GFP_KERNEL);

	/* Tell the handler to just drop the request. It'll be
	 * resubmitted later, hopefully, and we'll have decided
	 * whether to authenticate it by then.
	 */
	return NFSERR__DROPIT;
}
